%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    This function describes stochastic simulations of gene activity.     %
%    Imputs include states of A20 and IkBa genes as well as the amounts   %
%    activating (nuclear NF-kB) and repressing (nuclear IkBa) molecules.  %
%    Function returns time index (mk) of the change of gene activity      %
%    and new gene states.                                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [mk,G1a,G2a,G1,G2]=statuschange(G1ax,G2ax,G1x,G2x,Yact,Yin)
G1a=G1ax;G2a=G2ax;G1=G1x;G2=G2x;    %definition of local variables describing gene activity
NFparameters;
%   Yact- amount of NFkBn -Y(:,7) 
%   Yin - amount of IKBa  -Y(:,11)

%##########################################################################
%      Swithing time distribution  
ro=(2-G1ax-G2ax)*q1a*Yact+(G1ax+G2ax)*(q0a+q2a*Yin)+(2-G1x-G2x)*q1*Yact+(G1x+G2x)*(q0+q2*Yin); %total propensity function
fd=dt*cumtrapz(ro);        %integrated propensity function
fd=1-exp(-fd);             %Swithing time distribution (cdf)

%      randomization of the swithing time based on its distribution
r=rand;                    % uniform (0,1) random variable                     
if (fd(length(fd))<r)
    fprintf('Error. One may need to increase time interval for computing Risk Function\n');
    fprintf('since r > Risk Function at the end of the time interval.\n');
    fprintf('Risk function attains value %2.10f.\n',fd(length(fd)));    
end;

a=abs(fd-r);                    %invertion of the stwithing time cdf to find mk such that r=fd(mk)
mk=find((a-min(a))==0);         %mk = index (time) when the ststus changes 
clear a fd ro;

%##########################################################################
%   Randomization of gene activity according to the propensity functions
%   p1a-risk of attaching to IKBa site at time mk ---- p1a1, p1a2 -1st and 2nd allele, respectively
%   p2a-risk of deataching from IkBa at time mk
%   p1-risk of attaching to A20 site at time mk
%   p2-risk of deataching from A20 at time mk
p1a1=(1-G1ax)*q1a*Yact(mk);       % risk calculated from the propensity functions at t=mk
p2a1=G1ax*(q0a+q2a*Yin(mk));
p1a2=(1-G2ax)*q1a*Yact(mk);
p2a2=G2ax*(q0a+q2a*Yin(mk));
p11=(1-G1x)*q1*Yact(mk);
p21=G1x*(q0+q2*Yin(mk));
p12=(1-G2x)*q1*Yact(mk);
p22=G2x*(q0+q2*Yin(mk));

ss=(p1a1+p1a2+p2a1+p2a2+p11+p12+p21+p22);
p1a1=p1a1/ss;p2a1=p2a1/ss;        %normalized risks
p1a2=p1a2/ss;p2a2=p2a2/ss;
p11=p11/ss;p21=p21/ss;
p12=p12/ss;p22=p22/ss;

%     randomization of gene activity
rnumber=rand;                      %uniform random variable
if (rnumber<p1a1)                                        G1a=1;end;       %IKBa 1st allele (activation)
if (rnumber>=p1a1)&(rnumber<p1a1+p2a1)                   G1a=0;end;       %IKBa 1st allele (repression)  
if (rnumber>=p1a1+p2a1)&(rnumber<p1a1+p2a1+p1a2)             G2a=1;end;         %IKBa 2nd allele (activation)
if (rnumber>=p1a1+p2a1+p1a2)&(rnumber<p1a1+p2a1+p1a2+p2a2)         G2a=0;end;   %IKBa 2nd allele (repression)
if (rnumber>=p1a1+p2a1+p1a2+p2a2)&(rnumber<p1a1+p2a1+p1a2+p2a2+p11)     G1=1;end;       %A20 1st allele (activation)           
if (rnumber>=p1a1+p2a1+p1a2+p2a2+p11)&(rnumber<p1a1+p2a1+p1a2+p2a2+p11+p21)   G1=0;end  %A20 1st allele (deactivation)
if (rnumber>=p1a1+p2a1+p1a2+p2a2+p11+p21) &(rnumber<p1a1+p2a1+p1a2+p2a2+p11+p21+p12)  G2=1;end;    %A20 2nd allele (activation)
if (rnumber>=p1a1+p2a1+p1a2+p2a2+p11+p21+p12)   G2=0;end;                                          %A20 1nd allele (deactivation)